import datetime
import json
import logging
from assistant.config import CONFIG
import requests
from telegram.ext import Updater, CommandHandler, CallbackContext

log = logging.getLogger(__name__)


class NXFetchError(Exception):
    pass


def help_command(update, context: CallbackContext):
    """[help命令]

    Args:
        update ():
        context (CallbackContext]): [bot context]
    """
    context.bot.send_message(chat_id=update.effective_chat.id,
                             text='''
    指南：
    /nxcx - 检测今天是否打卡
    /chat_id - 检测当前聊天id(仅限管理员)
    /spider - 爬虫管理
    ''')
    log.info('help 命令 执行')


def check_clock_in_command(update, context: CallbackContext):
    if _remind_clock_in():
        context.bot.send_message(chat_id=update.effective_chat.id, text="已经打卡了！")
    else:
        context.bot.send_message(chat_id=update.effective_chat.id, text="还没有打卡！")


def check_chat_id_command(update, context: CallbackContext):
    chat_id = update.effective_chat.id
    context.bot.send_message(chat_id=chat_id, text=f'当前chat_id: {chat_id}')


def remind_clock_in_schedule(context: CallbackContext):
    """[宁夏出行打卡提醒]

    Args:
        context (CallbackContext]): [bot_context]
    """
    try:
        if not _remind_clock_in():
            context.bot.send_message(chat_id=CONFIG.announce_channel_id, text='我的宁夏 今天没有打卡哦')

    except Exception as e:
        log.error("我的宁夏打卡信息获取失败", e)
        context.bot.send_message(chat_id=CONFIG.announce_channel_id, text='我的宁夏打卡信息获取失败！！')


def _remind_clock_in():
    headers = {
        'user-agent': 'Mozilla/5.0 (Linux; Android 10; Redmi K30 5G Build/QKQ1.191222.002; wv) AppleWebKit/537.36 ('
                      'KHTML, like Gecko) Version/4.0 Chrome/84.0.4147.111 Mobile Safari/537.36 '
                      'wdnx-android_v1.21.0.0/1.0 ',
        'Content-Type': 'application/json'
    }
    post_data = {"userId": "2a1e10b9-896b-4233-a263-c6bb7e109a1b"}
    url = 'https://hcp.zwfw.nx.gov.cn/gjjkm/business/api/query/records?_t=1597228066&cid='
    response = requests.post(url,
                             data=json.dumps(post_data), verify=False, headers=headers)
    if response.status_code == 200:
        j = response.json()
        return _is_clock_in(j)
    else:
        log.error('我的宁夏打卡信息获取失败')
        raise NXFetchError('我的宁夏打卡信息获取失败！')


def _is_clock_in(res: dict):
    """判断今天的日期是否在已打卡数据中

    :param res 接收到的打卡信息，字典形式:
    :return: bool
    """
    for result in res['result']:
        today = str(datetime.date.today())
        if result['time'] == today:
            log.info('已经打卡了: ' + today)
            return True
        else:
            return False


def init_bot(token: str, proxy: dict = None):
    """
    初始化机器人
    :param token: 机器人密钥
    :param proxy: 本地socks代理
    :return:
    """
    updater = Updater(token=token, use_context=True,
                      request_kwargs=proxy)
    dispatcher = updater.dispatcher
    handlers = {
        'help_handler': CommandHandler('help', help_command),
        'check_clock_in': CommandHandler('nxcx', check_clock_in_command),
        'chat_id': CommandHandler('chat_id', check_chat_id_command)
    }
    for h in handlers:
        dispatcher.add_handler(handlers[h])
    job = updater.job_queue
    job.run_repeating(remind_clock_in_schedule, interval=60 * 60 * 6, first=datetime.datetime(2020, 8, 14))
    updater.start_polling()



import logging
import time
import re
from assistant.spider.request import get_soup

log = logging.getLogger(__name__)


class ShtSite:
    base_url = 'www.sehuatang.org'
    tags = {
        'cnSubVideo': '中文字幕',
    }


class CnSubtitleBoard:
    """ 中文字幕板块 """

    def __init__(self, url):
        self._tmp_content = get_soup(url)
        self.page_num = self.get_page_num()  # 当前页码
        self.page_movie_list = self.get_page_movie_list()  # 当前页码url
        self.movie_num = 0  # 电影总数
        self.next_page = self.get_next_page()

    def get_page_num(self):
        try:
            page_num = self._tmp_content.select_one('div.pg strong').text
            log.debug(f'page num : {page_num}')
            return int(page_num)
        except Exception as e:
            log.error('page num parsing failed!!', e)

    def get_next_page(self):
        try:
            next_page = self._tmp_content.select_one('div.pg a.nxt')['href']
            log.debug(f'next page : {next_page}')
            return next_page
        except Exception as e:
            log.error('next page parsing failed!!', e)

    def get_page_movie_list(self):
        try:
            li = [thread.select_one('a[href^=thread]')['href']
                  for thread in self._tmp_content.select('tbody[id^="normalthread_"]')]
            log.debug(f'page movie list : {li}')
            return li
        except Exception as e:
            log.error(f'page movie list parsing failed!!', e)

    def start_crawl(self):
        while self.next_page:
            log.info('collecting cnSub index page: '+self.next_page)
            log.info(self.page_num)
            # for detail_page in self.page_movie_list:
            #     detail = CnSubtitleThread(f'http://{ShtSite.base_url}/{detail_page}')


class CnSubtitleThread:
    """ 中文字幕电影 """

    def __init__(self, url):
        log.debug('=' * 16)
        log.debug(f'crawling {url}')
        self.tmp_content = get_soup(url)
        self.post_id = self.get_post_id()
        self.title = self.get_title()
        self.url = url
        self.actor = self.get_actor()
        self.post_time = self.get_post_time()
        self.mag_link = self.get_mag_link()
        self.tag = self.get_tag()
        self.heat = self.get_heat()
        log.debug('=' * 16)
        log.info(self.__str__())

    def __str__(self):
        info = '\n========= cn sub video info ========= \n'
        for k, v in self.__dict__.items():
            if k != 'tmp_content':
                if not k == 'post_time':
                    info += f'{k:<10} : {v:<100} \n'
                else:
                    info += f'{k:<10} : {time.strftime("%Y-%m-%d %H:%M:%S", v):<100} \n'
        info += '======================================='
        return info

    def get_post_id(self):
        try:
            post_id = str.split(self.tmp_content.select_one('div[id^=post_]')['id'], '_')[-1]
            log.debug(f'post id : {post_id}')
            return post_id
        except Exception as e:
            log.error("post id parsing failed", e)

    def get_title(self):
        try:
            title = self.tmp_content.select_one('#thread_subject').text
            log.debug(f"title: {title}")
            return title
        except Exception as e:
            log.error('title parsing failed!!', e)

    def get_post_time(self):
        try:
            time_str = self.tmp_content.select_one(f'#authorposton{self.post_id} span')['title']
            post_time = time.strptime(time_str, '%Y-%m-%d %H:%M:%S')
            log.debug(f'post time : {time.strftime("%Y-%m-%d %H:%M:%S", post_time)}')
            return post_time
        except Exception as e:
            log.error('post time parsing failed!!', e)

    def get_tag(self):
        try:
            tag = self.tmp_content.select_one('h1.ts a').text.replace('[', '').replace(']', '')
            log.debug(f'tag : {tag}')
            return tag
        except Exception as e:
            log.error('tag parsing failed!!', e)

    def get_mag_link(self):
        try:
            mag_link = self.tmp_content.select_one('div[id^="code_"] li').text
            log.debug(f'mag link : {mag_link}')
            return mag_link
        except Exception as e:
            log.error('mag link parsing error!!', e)

    def get_heat(self):
        try:
            heat = self.tmp_content.select_one('span.xi1').text
            log.debug(f'heat : {heat}')
            return int(heat)
        except Exception as e:
            log.error('heat parsing error!!', e)

    def get_actor(self):
        try:
            post = self.tmp_content.select_one(f'#postmessage_{self.post_id}').text.split('\n')
            actor = [line.split("：")[-1].strip() for line in post if re.search("女优", line)][0]
            log.debug(f'actor : {actor}')
            return actor
        except Exception as e:
            log.error('actor parsing error', e)


if __name__ == '__main__':
    pass



from assistant.bot import assistant_bot


def test__is_clock_in():
    payload = {'code': '0', 'msg': '成功', 'result': [{'time': '2020-08-15', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-14', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-13', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-12', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-11', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-10', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-09', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-08', 'conditions': ['正常状态'], 'tempature': '36 ℃'},
                                                    {'time': '2020-08-06', 'conditions': ['正常状态'], 'tempature': '36 ℃'}
                                                    ]}

    res = assistant_bot._is_clock_in(payload)
    assert res


def test__remind_clockin():
    assistant_bot._remind_clockin()



import logging
from assistant.bot import assistant_bot
from assistant.spider import sht
from assistant.config import CONFIG

log = logging.getLogger(__name__)


def start_crawl():
    board = sht.CnSubtitleBoard('https://www.sehuatang.org/forum-103-1.html')


def main():
    logging.basicConfig(level=CONFIG.log_level, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    assistant_bot.init_bot(CONFIG.bot_token, CONFIG.proxy())


if __name__ == '__main__':
    main()



from bs4 import BeautifulSoup
import requests
import logging
import os
from urllib.parse import urlparse
from assistant.config import CONFIG

proxy_site = ['sehuatang.org']
log = logging.getLogger(__name__)


def get_soup(url):
    file_name = urlparse(url).path
    if not file_name:
        file_name = f'{urlparse(url).netloc}.html'
    buffer_path = os.path.join(os.path.dirname(CONFIG.basedir), 'cache', file_name)
    if not os.path.exists(buffer_path):
        log.debug(f'缓存路径:{buffer_path}')
        content = _get_content_http(url)
        with open(buffer_path, 'w', encoding='utf-8') as f:
            f.write(content.decode('utf-8'))
        return BeautifulSoup(content, 'html.parser')
    else:
        log.debug('buffer found，open file')
        return BeautifulSoup(open(buffer_path, encoding='utf-8'), 'html.parser')


def _get_content_http(url):
    """ :return  soup"""
    for site in proxy_site:
        # 检测是否在需要代理的范围中
        if site in url:
            log.debug(f'site need proxy: {url}')
            return requests.get(url, proxies=CONFIG.proxy(tg=False)).content
        else:
            log.debug('starting http request with no proxy')
            return requests.get(url).content




from setuptools import setup, find_packages

setup(
    name='jacob-assistant',
    packages=find_packages(),
    version='0.1.2',
    author='jacob',
    description="jacob的个人助手",
    keywords='jacob-assistant assistant',
    install_requires=[
        'beautifulsoup4==4.9.1',
        'requests==2.24.0',
        'Flask==1.1.2',
        'python-telegram-bot==12.8'
    ],
    data_files=[
        ('assistant', ['assistant/conf/dev.ini', 'assistant/conf/prod.ini'])
    ],
    entry_points={
        'console_scripts': [
            'assistant=assistant.main:main'
        ]
    }
)



import os
import configparser
import logging
import sys

"""
项目配置
"""
log = logging.getLogger(__name__)

_log_level = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warn': logging.WARN,
    'error': logging.ERROR
}


class BaseConfig:
    """
    配置类，根据环境变量 'assistant_env' 切换开发环境(prod)与部署环境(dev)配置文件
    """

    def __init__(self):
        try:
            self.env = os.getenv('assistant_env', 'dev')
            self.basedir = os.path.dirname(__file__)
            self.config_path = self.conf_path()
            config = configparser.ConfigParser()
            config.read(self.config_path)

            self.bot_token = config.get('telegram', 'bot_token')
            self.proxy_on = config.getboolean('proxy', 'proxy')
            self.proxy_url = config.get('proxy', 'url')
            self.log_level = _log_level[config['general']['log_level']]
            self.announce_channel_id = config.get('telegram', 'announce_channel_id')
            self.admin_id = config.get('telegram', 'admin_id')

        except Exception as e:
            log.critical('配置文件读取失败，退出', e)
            log.error('配置文件路径: ' + self.config_path)

    def conf_path(self, path=None):
        """
        :param path: 命令行输入的路径
        :return: 根据生产环境和开发环境返回路径
        """
        if path:
            return path
        else:
            if self.env == 'prod':
                default_path = '/etc/jacob-assistant/prod.ini'
                if os.path.exists(default_path):
                    return default_path
                else:
                    log.critical('配置文件未找到 请配置 /etc/jacob-assistant/prod.ini')
                    sys.exit(2)
            else:
                # 开发环境配置文件路径
                return os.path.join(self.basedir, 'conf', 'dev.ini')

    def proxy(self, tg=True):
        if self.proxy_on:
            if tg:
                # 给TG的代理
                return {'proxy_url': self.proxy_url}
            else:
                # 给requests的代理
                return {
                    'http': self.proxy_url,
                    'https': self.proxy_url
                }


CONFIG = BaseConfig()
